<?php
/**
 * @file Primarily handles the return flow from Amazon Payments.
 */

// Handles a complete Flexible Payments Service sale.
function mp_amazon_fps_complete($order) {
  // Verify that Amazon Payments issued the redirect to the completion URL.
  module_load_include('inc', 'uc_amazonpayments', 'uc_amazonpayments.pages');
  $payment_authorized = uc_amazonpayments_verify_signature();
  if ($payment_authorized) {
    module_load_include('inc', 'mp_amazon_fps', 'mp_amazon_fps.payment');
    // Store the multi-use token in the order data
    $order->data['mp_amazon_fps_multi_use_token'] = check_plain($_GET['tokenID']);
    uc_order_save($order);

    // Retrieve the order total per seller
    $order_sellers = _mp_amazon_fps_order_sellers($order);

    foreach ($order_sellers as $uid => $seller) {
      // Issue a Pay request to Amazon Payments. This will also transmit an IPN
      // to the site.
      // Build the request to send to the Amazon FPS API
      $authorization = mp_amazon_fps_authorization_load($uid);
      $caller_reference = check_plain($_GET['callerReference']) . "-{$uid}";
      $sender_token_id = check_plain($_GET['tokenID']);
      $recipient_token_id = $authorization->recipient_token_id;
      $transaction_amount_value = $seller->seller_total;
      $fps = mp_amazon_fps_pay_seller($order, $seller, $caller_reference,
        $sender_token_id, $recipient_token_id, $transaction_amount_value);
    }
  }
  _uc_amazonpayments_complete($order, $payment_authorized);
}

function mp_amazon_fps_pay_seller($order, $seller, $caller_reference, $sender_token_id,
    $recipient_token_id, $amount) {
  amazon_fps_include('fps');
  module_load_include('inc', 'uc_amazonpayments', 'uc_amazonpayments.pages');
  $fps_config = array('ServiceURL' => _uc_amazonpayments_fps_api_url());
  $request = array(
    'CallerReference' => $caller_reference,
    'SenderTokenId' => $sender_token_id,
    'RecipientTokenId' => $recipient_token_id,
    'TransactionAmount' => array(
      'CurrencyCode' => 'USD',
      'Value' => $amount,
    ),
    'OverrideIPNURL' => url('uc_amazonpayments/ipn',
      array('absolute' => TRUE)),
    );
  // Allow other modules to change the contents of the Pay request
  drupal_alter('mp_amazon_fps_payment', $request, $order, $seller);
  $fps = amazon_fps_fps_pay($request, $fps_config);
  return $fps;
}

