<?php
/**
 * @file
 * Contains functions that integrate Amazon FPS seller payment with other
 *   modules. Hook implementations are in the .module file, but the helper
 *   functions that do the work are here.
 */

function mp_amazon_fps_checkout_review_form_submit($form, &$form_state) {
  // Load the order
  $order = uc_order_load($form_state['values']['order_id']);

  // Determine how many uses the token will need (number of sellers)
  $order_sellers = _mp_amazon_fps_order_sellers($order);
  $seller_count = count($order_sellers);
  // @todo: Make sure this works when I implement reserving payments.
  /* @todo: Probably some code to find sellers who don't take full
    payment at once would go here. Then I could add the counts together. */

  // Get the most expensive charge (this is the maximum transaction amount)
  $highest_price = _mp_amazon_fps_order_highest_price($order);

  $seller_recipient_tokens = _mp_amazon_fps_seller_recipient_tokens(
    array_keys($order_sellers));

  // Build the URL and set $form_state['redirect'] to it.
  amazon_fps_include('cbui_multi_use');
  $cbui = amazon_fps_cbui_multi_use(array(
    'caller_reference' => $form_state['values']['callerReference'],
    'return_url' => url("mp_amazon_fps/complete/{$form_state['values']['order_id']}",
      array('absolute' => TRUE)),
    'global_amount_limit' => $form_state['values']['amount_1'],
    ),
    array(
      'currencyCode' => 'USD',
      'paymentReason' => $form_state['values']['item_name_1'],
      'transactionAmount' => $highest_price,
      'amountType' => 'Maximum',
    ),
    TRUE, // The TRUE asks the function for the URL directly.
    variable_get('uc_amazonpayments_fps_server',
      'https://authorize.payments-sandbox.amazon.com/cobranded-ui/actions/start'),
    NULL, // UsageLimit1
    NULL, // UsageLimit2
    FALSE, // IsCobranding
    $seller_recipient_tokens); // RecipientTokenList

  // Set the form to go there at redirect time.
  $form_state['redirect'] = $cbui;
}

/**
 * Get a list of loaded accounts for sellers whose products
 *   are in this order.
 */
function _mp_amazon_fps_order_sellers($order) {
  $sellers = array();
  foreach ($order->products as $product) {
    $product_node = node_load($product->nid);
    // We index by uid to prevent duplicates
    $sellers[$product_node->uid] = user_load($product_node->uid);
    if (!isset($sellers[$product_node->uid]->seller_total)) {
      $sellers[$product_node->uid]->seller_total = 0.0;
    }
    // Add the current product object to the seller's products
    if (!isset($sellers[$product_node->uid]->products)) {
      $sellers[$product_node->uid]->products = array();
    }
    $sellers[$product_node->uid]->products[] = $product;
    $index = count($sellers[$product_node->uid]->products) - 1;
    $sellers[$product_node->uid]->products[$index]->product_total = $product->price * $product->qty;

    // Add in the current product's total price to this seller's order total
    // @todo: Debug this calculation; something seems wrong
    $sellers[$product_node->uid]->seller_total += $sellers[$product_node->uid]->products[$index]->product_total;
  }
  return $sellers;
}

/**
 * I don't really have to explain this one, right?
 */
function _mp_amazon_fps_order_highest_price($order) {
  $highest_price = 0.0;
  foreach ($order->products as $product) {
    if (($product->price * $product->qty) > $highest_price) {
      $highest_price = $product->price * $product->qty;
    }
  }
  return $highest_price;
}

/**
 * Turn a list of uids into a list of recipient tokens.
 */
function _mp_amazon_fps_seller_recipient_tokens($uids) {
  $recipient_tokens = array();
  foreach ($uids as $uid) {
    $authorization = mp_amazon_fps_authorization_load($uid);
    if ($authorization) {
      $recipient_tokens[] = $authorization->recipient_token_id;
    }
  }
  return $recipient_tokens;
}

